/*
 * tegra210_ope.h - Tegra OPE driver header.
 *
 * Author: Sumit Bhattacharya <sumitb@nvidia.com>
 *
 * Copyright (C) 2013-2014, NVIDIA CORPORATION. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#ifndef __TEGRA210_MBDRC_H__
#define __TEGRA210_MBDRC_H__

#include "tegra210_ahub_utils.h"

#define TEGRA210_MBDRC_COUNT		2
#define TEGRA210_MBDRC_MAX_BIQ_STAGES	8

/* Order of these enums are same as the order of band specific hw registers */
enum {
	MBDRC_LOW_BAND,
	MBDRC_MID_BAND,
	MBDRC_HIGH_BAND,
	MBDRC_MAX_BAND,
};

/* Register offsets from TEGRA210_MBDRC_BASE */
#define TEGRA210_MBDRC_SOFT_RST			0X4
#define TEGRA210_MBDRC_CG			0X8
#define TEGRA210_MBDRC_STATUS			0Xc
#define TEGRA210_MBDRC_CONFIG			0X28
#define TEGRA210_MBDRC_CHAN_MASK		0X2c
#define TEGRA210_MBDRC_MASTER_VOLUME		0X30
#define TEGRA210_MBDRC_FAST_FACTOR		0X34

#define TEGRA210_MBDRC_FILTER_COUNT		3
#define TEGRA210_MBDRC_FILTER_PARAM_STRIDE	0x4

#define TEGRA210_MBDRC_IIR_CONFIG		0X38
#define TEGRA210_MBDRC_IN_ATTACK		0x44
#define TEGRA210_MBDRC_IN_RELEASE		0X50
#define TEGRA210_MBDRC_FAST_ATTACK		0X5c
#define TEGRA210_MBDRC_IN_THRESH		0X68
#define TEGRA210_MBDRC_OUT_THRESH		0X74
#define TEGRA210_MBDRC_RATIO_1ST		0X80
#define TEGRA210_MBDRC_RATIO_2ND		0X8c
#define TEGRA210_MBDRC_RATIO_3RD		0X98
#define TEGRA210_MBDRC_RATIO_4TH		0Xa4
#define TEGRA210_MBDRC_RATIO_5TH		0Xb0
#define TEGRA210_MBDRC_MAKEUP_GAIN		0Xbc
#define TEGRA210_MBDRC_INIT_GAIN		0Xc8
#define TEGRA210_MBDRC_GAIN_ATTACK		0Xd4
#define TEGRA210_MBDRC_GAIN_RELEASE		0Xe0
#define TEGRA210_MBDRC_FAST_RELEASE		0Xec
#define TEGRA210_MBDRC_AHUBRAMCTL_MBDRC_CTRL	0Xf8
#define TEGRA210_MBDRC_AHUBRAMCTL_MBDRC_DATA	0X104

#define TEGRA210_MBDRC_MAX_REG			(TEGRA210_MBDRC_AHUBRAMCTL_MBDRC_DATA + \
						(TEGRA210_MBDRC_FILTER_PARAM_STRIDE * \
						(TEGRA210_MBDRC_FILTER_COUNT - 1)))
/* Fields for TEGRA210_MBDRC_SOFT_RST */
#define TEGRA210_MBDRC_SOFT_RST_RESET			BIT(0)

/* Fields for TEGRA210_MBDRC_CG */
#define TEGRA210_MBDRC_CG_SLCG_EN			BIT(0)

/* Fields for TEGRA210_MBDRC_STATUS */
#define TEGRA210_MBDRC_STATUS_CONFIG_ERR		BIT(31)
#define TEGRA210_MBDRC_STATUS_SLCG_CLKEN		BIT(0)

/* Fields for TEGRA210_MBDRC_CONFIG */
#define TEGRA210_MBDRC_CONFIG_RMS_OFFSET_SHIFT		16
#define TEGRA210_MBDRC_CONFIG_RMS_OFFSET_MASK		(0x1ff << TEGRA210_MBDRC_CONFIG_RMS_OFFSET_SHIFT)

#define TEGRA210_MBDRC_CONFIG_BIAS_UNBIAS		BIT(15)

#define TEGRA210_MBDRC_CONFIG_PEAK_RMS_SHIFT		14
#define TEGRA210_MBDRC_CONFIG_PEAK_RMS_MASK		(0x1 << TEGRA210_MBDRC_CONFIG_PEAK_RMS_SHIFT)

#define MBDRC_RMS					0
#define MBDRC_PEAK					1

#define TEGRA210_MBDRC_CONFIG_PEAK_RMS_RMS		(MBDRC_RMS << TEGRA210_MBDRC_CONFIG_PEAK_RMS_SHIFT)
#define TEGRA210_MBDRC_CONFIG_PEAK_RMS_PEAK		(MBDRC_PEAK << TEGRA210_MBDRC_CONFIG_PEAK_RMS_SHIFT)

#define TEGRA210_MBDRC_CONFIG_FILT_STRUC_SHIFT		13
#define TEGRA210_MBDRC_CONFIG_FILT_STRUC_MASK		(0x1 << TEGRA210_MBDRC_CONFIG_FILT_STRUC_SHIFT)

#define MBDRC_FILT_ALLPASS				0
#define MBDRC_FILT_FLEX					1

#define TEGRA210_MBDRC_CONFIG_FILT_STRUC_ALL_PASS_TREE	(MBDRC_FILT_ALLPASS << TEGRA210_MBDRC_CONFIG_FILT_STRUC_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FILT_STRUC_FLEX		(MBDRC_FILT_FLEX << TEGRA210_MBDRC_CONFIG_FILT_STRUC_SHIFT)

#define TEGRA210_MBDRC_CONFIG_SHIFT_CTRL_SHIFT		8
#define TEGRA210_MBDRC_CONFIG_SHIFT_CTRL_MASK		(0x1f << TEGRA210_MBDRC_CONFIG_SHIFT_CTRL_SHIFT)

#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT		4
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_MASK		(0xf << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)

#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N1		(0 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N2		(1 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N4		(2 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N8		(3 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N16		(4 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N32		(5 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_FRAME_SIZE_N64		(6 << TEGRA210_MBDRC_CONFIG_FRAME_SIZE_SHIFT)

#define TEGRA210_MBDRC_CONFIG_MBDRC_MODE_SHIFT		0
#define TEGRA210_MBDRC_CONFIG_MBDRC_MODE_MASK		(0x3 << TEGRA210_MBDRC_CONFIG_MBDRC_MODE_SHIFT)

#define MBDRC_MODE_BYPASS				0
#define MBDRC_MODE_FULLBAND				1
#define MBDRC_MODE_DUALBAND				2
#define MBDRC_MODE_MULTIBAND				3

#define TEGRA210_MBDRC_CONFIG_MBDRC_MODE_BYPASS		(MBDRC_MODE_BYPASS << TEGRA210_MBDRC_CONFIG_MBDRC_MODE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_MBDRC_MODE_FULLBAND	(MBDRC_MODE_FULLBAND << TEGRA210_MBDRC_CONFIG_MBDRC_MODE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_MBDRC_MODE_DUALBAND	(MBDRC_MODE_DUALBAND << TEGRA210_MBDRC_CONFIG_MBDRC_MODE_SHIFT)
#define TEGRA210_MBDRC_CONFIG_MBDRC_MODE_MULTIBAND	(MBDRC_MODE_MULTIBAND << TEGRA210_MBDRC_CONFIG_MBDRC_MODE_SHIFT)

/* Fields for TEGRA210_MBDRC_CHAN_MASK */
#define TEGRA210_MBDRC_CHAN_MASK_SHIFT			0
#define TEGRA210_MBDRC_CHAN_MASK_MASK			(0xff << TEGRA210_MBDRC_CHAN_MASK_SHIFT)

/* Fields for TEGRA210_MBDRC_MASTER_VOLUME */
#define TEGRA210_MBDRC_MASTER_VOLUME_SHIFT		0
#define TEGRA210_MBDRC_MASTER_VOLUME_MASK		(0xffffffff << TEGRA210_MBDRC_MASTER_VOLUME_SHIFT)

/* Fields for TEGRA210_MBDRC_FAST_FACTOR */
#define TEGRA210_MBDRC_FAST_FACTOR_FR_FACTOR_SHIFT	16
#define TEGRA210_MBDRC_FAST_FACTOR_FR_FACTOR_MASK	(0xffff << TEGRA210_MBDRC_FAST_FACTOR_FR_FACTOR_SHIFT)

#define TEGRA210_MBDRC_FAST_FACTOR_FA_FACTOR_SHIFT	0
#define TEGRA210_MBDRC_FAST_FACTOR_FA_FACTOR_MASK	(0xffff << TEGRA210_MBDRC_FAST_FACTOR_FA_FACTOR_SHIFT)

/* Fields for TEGRA210_MBDRC_IIR_CONFIG */
#define TEGRA210_MBDRC_IIR_CONFIG_NUM_STAGES_SHIFT	0
#define TEGRA210_MBDRC_IIR_CONFIG_NUM_STAGES_MASK	(0xf << TEGRA210_MBDRC_IIR_CONFIG_NUM_STAGES_SHIFT)

/* Fields for TEGRA210_MBDRC_IN_ATTACK */
#define TEGRA210_MBDRC_IN_ATTACK_TC_SHIFT		0
#define TEGRA210_MBDRC_IN_ATTACK_TC_MASK		(0xffffffff << TEGRA210_MBDRC_IN_ATTACK_TC_SHIFT)

/* Fields for TEGRA210_MBDRC_IN_RELEASE */
#define TEGRA210_MBDRC_IN_RELEASE_TC_SHIFT		0
#define TEGRA210_MBDRC_IN_RELEASE_TC_MASK		(0xffffffff << TEGRA210_MBDRC_IN_RELEASE_TC_SHIFT)

/* Fields for TEGRA210_MBDRC_FAST_ATTACK */
#define TEGRA210_MBDRC_FAST_ATTACK_TC_SHIFT		0
#define TEGRA210_MBDRC_FAST_ATTACK_TC_MASK		(0xffffffff << TEGRA210_MBDRC_FAST_ATTACK_TC_SHIFT)

/* Fields for TEGRA210_MBDRC_IN_THRESH */
#define TEGRA210_MBDRC_IN_THRESH_4TH_SHIFT		24
#define TEGRA210_MBDRC_IN_THRESH_4TH_MASK		(0xff << TEGRA210_MBDRC_IN_THRESH_4TH_SHIFT)

#define TEGRA210_MBDRC_IN_THRESH_3RD_SHIFT		16
#define TEGRA210_MBDRC_IN_THRESH_3RD_MASK		(0xff << TEGRA210_MBDRC_IN_THRESH_3RD_SHIFT)

#define TEGRA210_MBDRC_IN_THRESH_2ND_SHIFT		8
#define TEGRA210_MBDRC_IN_THRESH_2ND_MASK		(0xff << TEGRA210_MBDRC_IN_THRESH_2ND_SHIFT)

#define TEGRA210_MBDRC_IN_THRESH_1ST_SHIFT		0
#define TEGRA210_MBDRC_IN_THRESH_1ST_MASK		(0xff << TEGRA210_MBDRC_IN_THRESH_1ST_SHIFT)

/* Fields for TEGRA210_MBDRC_OUT_THRESH */
#define TEGRA210_MBDRC_OUT_THRESH_4TH_SHIFT		24
#define TEGRA210_MBDRC_OUT_THRESH_4TH_MASK		(0xff << TEGRA210_MBDRC_OUT_THRESH_4TH_SHIFT)

#define TEGRA210_MBDRC_OUT_THRESH_3RD_SHIFT		16
#define TEGRA210_MBDRC_OUT_THRESH_3RD_MASK		(0xff << TEGRA210_MBDRC_OUT_THRESH_3RD_SHIFT)

#define TEGRA210_MBDRC_OUT_THRESH_2ND_SHIFT		8
#define TEGRA210_MBDRC_OUT_THRESH_2ND_MASK		(0xff << TEGRA210_MBDRC_OUT_THRESH_2ND_SHIFT)

#define TEGRA210_MBDRC_OUT_THRESH_1ST_SHIFT		0
#define TEGRA210_MBDRC_OUT_THRESH_1ST_MASK		(0xff << TEGRA210_MBDRC_OUT_THRESH_1ST_SHIFT)

/* Fields for TEGRA210_MBDRC_RATIO_1ST */
#define TEGRA210_MBDRC_RATIO_1ST_SHIFT			0
#define TEGRA210_MBDRC_RATIO_1ST_MASK			(0xffff << TEGRA210_MBDRC_RATIO_1ST_SHIFT)

/* Fields for TEGRA210_MBDRC_RATIO_2ND */
#define TEGRA210_MBDRC_RATIO_2ND_SHIFT			0
#define TEGRA210_MBDRC_RATIO_2ND_MASK			(0xffff << TEGRA210_MBDRC_RATIO_2ND_SHIFT)

/* Fields for TEGRA210_MBDRC_RATIO_3RD */
#define TEGRA210_MBDRC_RATIO_3RD_SHIFT			0
#define TEGRA210_MBDRC_RATIO_3RD_MASK			(0xffff << TEGRA210_MBDRC_RATIO_3RD_SHIFT)

/* Fields for TEGRA210_MBDRC_RATIO_4TH */
#define TEGRA210_MBDRC_RATIO_4TH_SHIFT			0
#define TEGRA210_MBDRC_RATIO_4TH_MASK			(0xffff << TEGRA210_MBDRC_RATIO_4TH_SHIFT)

/* Fields for TEGRA210_MBDRC_RATIO_5TH */
#define TEGRA210_MBDRC_RATIO_5TH_SHIFT			0
#define TEGRA210_MBDRC_RATIO_5TH_MASK			(0xffff << TEGRA210_MBDRC_RATIO_5TH_SHIFT)

/* Fields for TEGRA210_MBDRC_MAKEUP_GAIN */
#define TEGRA210_MBDRC_MAKEUP_GAIN_SHIFT		0
#define TEGRA210_MBDRC_MAKEUP_GAIN_MASK			(0x3f << TEGRA210_MBDRC_MAKEUP_GAIN_SHIFT)

/* Fields for TEGRA210_MBDRC_INIT_GAIN */
#define TEGRA210_MBDRC_INIT_GAIN_SHIFT			0
#define TEGRA210_MBDRC_INIT_GAIN_MASK			(0xffffffff << TEGRA210_MBDRC_INIT_GAIN_SHIFT)

/* Fields for TEGRA210_MBDRC_GAIN_ATTACK */
#define TEGRA210_MBDRC_GAIN_ATTACK_SHIFT		0
#define TEGRA210_MBDRC_GAIN_ATTACK_MASK			(0xffffffff << TEGRA210_MBDRC_GAIN_ATTACK_SHIFT)

/* Fields for TEGRA210_MBDRC_GAIN_RELEASE */
#define TEGRA210_MBDRC_GAIN_RELEASE_SHIFT		0
#define TEGRA210_MBDRC_GAIN_RELEASE_MASK		(0xffffffff << TEGRA210_MBDRC_GAIN_RELEASE_SHIFT)

/* Fields for TEGRA210_MBDRC_FAST_RELEASE */
#define TEGRA210_MBDRC_FAST_RELEASE_SHIFT		0
#define TEGRA210_MBDRC_FAST_RELEASE_MASK		(0xffffffff << TEGRA210_MBDRC_FAST_RELEASE_SHIFT)

/* Fields for TEGRA210_MBDRC_AHUBRAMCTL_MBDRC_CTRL defined in tegra210_ahub_utils.h */
/* Fields for TEGRA210_MBDRC_AHUBRAMCTL_MBDRC_DATA defined in tegra210_ahub_utils.h */

#define MBDRC_THRESH_MAX	4
#define MBDRC_RATIO_MAX		(MBDRC_THRESH_MAX + 1)

#define TEGRA210_MBDRC_BIQ_PARAMS_PER_STAGE		5

struct mbdrc_biq_params {
	s32 biq_b0;
	s32 biq_b1;
	s32 biq_b2;
	s32 biq_a1;
	s32 biq_a2;
};

struct tegra210_mbdrc_band_params {
	unsigned int band;
	unsigned int iir_stages;
	unsigned int in_attack_tc;
	unsigned int in_release_tc;
	unsigned int fast_attack_tc;
	unsigned int in_thresh[MBDRC_THRESH_MAX];
	unsigned int out_thresh[MBDRC_THRESH_MAX];
	unsigned int ratio[MBDRC_RATIO_MAX];
	unsigned int makeup_gain;
	unsigned int gain_init;
	unsigned int gain_attack_tc;
	unsigned int gain_release_tc;
	unsigned int fast_release_tc;
	struct mbdrc_biq_params biq_params[TEGRA210_MBDRC_MAX_BIQ_STAGES];
};

struct tegra210_mbdrc_config {
	unsigned int mode;
	unsigned int rms_off;
	unsigned int peak_rms_mode;
	unsigned int flit_struc;
	unsigned int shift_ctrl;
	unsigned int frame_size;
	unsigned int chan_mask;
	unsigned int fa_fact;	/* Fast attack factor */
	unsigned int fr_fact;	/* Fast release factor */
	struct tegra210_mbdrc_band_params band_params[MBDRC_MAX_BAND];
};

void tegra210_mbdrc_enable_clk(enum tegra210_ahub_cifs cif);
void tegra210_mbdrc_disable_clk(enum tegra210_ahub_cifs cif);
int tegra210_mbdrc_is_slcg_enable(enum tegra210_ahub_cifs cif);
int tegra210_mbdrc_is_config_err(enum tegra210_ahub_cifs cif);
int tegra210_mbdrc_reset(enum tegra210_ahub_cifs cif);
int tegra210_mbdrc_set_slcg(enum tegra210_ahub_cifs cif, int en);
int tegra210_mbdrc_set_bypass(enum tegra210_ahub_cifs cif);
int tegra210_mbdrc_set_config(enum tegra210_ahub_cifs cif,
			      struct tegra210_mbdrc_config *config);
int tegra210_mbdrc_set_master_vol(enum tegra210_ahub_cifs cif,
				  unsigned int vol);

#endif
